#include "OgrePathMapRenderTechnique.h"
#include "OgreTechniqueGroup.h"
#include "OgreIlluminationManager.h"
#include "OgrePMWeightComputeRenderingRun.h"



OgrePathMapRenderTechnique::OgrePathMapRenderTechnique(											
												Pass* pass,
												OgreRenderable* parentRenderable,
												OgreTechniqueGroup* parentTechniqueGroup)
							:RenderTechnique( parentRenderable, parentTechniqueGroup),
							OgreRenderTechnique(pass, parentRenderable, parentTechniqueGroup)
{	
	this->clusters = OgreIlluminationManager::getSingleton().getPathMapClusters(parentRenderable->getName());
	
	//insert new pass
	Ogre::Technique* techn = pass->getParent();
	Technique::PassIterator it = techn->getPassIterator();
	
	int index = 0;
	while(it.hasMoreElements())
	{
		if( it.getNext() == pass)
			break;
		index++;
		it.moveNext();
	}
	
	Pass* newpass = pathMapPass = techn->createPass();
	
	newpass->setVertexProgram("GTP/PathMap_VS");
	newpass->setFragmentProgram("GTP/PathMap_PS");
	
	//bind vertex program parameters
	GpuProgramParameters* Vparams = newpass->getVertexProgramParameters().getPointer();
	Vparams->setNamedAutoConstant("WorldViewProj", GpuProgramParameters::ACT_WORLDVIEWPROJ_MATRIX);	
	//bind fragment program parameters
	GpuProgramParameters* Fparams = newpass->getFragmentProgramParameters().getPointer();
	int prmxres = clusters->pathMapResolution * clusters->count;
	int prmyres = clusters->pathMapResolution;
	while(prmxres > 4096)
	{
		prmxres /= 2;
		prmyres *= 2;		
	}
	int prmnt[2] = {prmxres / clusters->pathMapResolution, prmyres / clusters->pathMapResolution};

	float halfPixel[2] = {0.5 / prmxres, 0.5 / prmyres};
	Vector4 pathMapParameters(prmnt[0],prmnt[1],halfPixel[0],halfPixel[1]);
	Fparams->setNamedConstant("prmAtlasTilesHalfPixel",pathMapParameters);
    		
	TextureUnitState* st = newpass->createTextureUnitState();		
	st->setTextureFiltering(TFO_BILINEAR);
	st->setTextureAddressingMode(TextureUnitState::TAM_BORDER);
	st->setTextureBorderColour(ColourValue::Blue);
	st->setTextureName(clusters->pathMapTextureFilename);
	
	//newpass->setSceneBlending(SBT_MODULATE);
	newpass->setSceneBlending(SBF_ONE, SBF_ZERO);
	newpass->setDepthBias(1);
	

	
}

OgrePathMapRenderTechnique::~OgrePathMapRenderTechnique()
{

}

void OgrePathMapRenderTechnique::update(unsigned long frameNum)
{	
	LightList lights;
	SceneManager* sm = Root::getSingleton()._getCurrentSceneManager();
	sm->_populateLightList(Vector3(0,0,0), 1000, lights); //TODO
//TODO set weights
	PathMapClusters* clusters = OgreIlluminationManager::getSingleton().getPathMapClusters(parentOgreRenderable->getName());
	float *actualWeights = new float[clusters->count];//TODO = 0
	for(int i = 0 ; i < 1; i++)
	{
		String lightName = lights.at(0)->getName();
		OgreIlluminationManager::getSingleton().createPerLightRun(lightName, ILLUMRUN_PM_WEIGHTMAP);
		OgreIlluminationManager::getSingleton().updatePerLightRun(lightName, ILLUMRUN_PM_WEIGHTMAP, frameNum);
		OgrePMWeightComputeRenderingRun* PMWeightRun = (OgrePMWeightComputeRenderingRun*)
			OgreIlluminationManager::getSingleton().getPerLightRun(lightName, ILLUMRUN_PM_WEIGHTMAP)->asOgreRenderingRun();

		float* weights = PMWeightRun->getWeights();
		
		for(int j = 0; j< clusters->count; j++)
			actualWeights[j] = weights[clusters->clusters[j]];
				
	}

	GpuProgramParameters* Fparams = pathMapPass->getFragmentProgramParameters().getPointer();
	Fparams->setNamedConstant("weights", actualWeights, clusters->count);
}

OgrePathMapRenderTechniqueFactory::OgrePathMapRenderTechniqueFactory()
{
	typeName = "PathMap";
}

OgreRenderTechnique* OgrePathMapRenderTechniqueFactory::createInstance( 
										IllumTechniqueParams* params,
										Pass* pass,
										OgreRenderable* parentRenderable,
										OgreTechniqueGroup* parentTechniqueGroup)
{	
	parseParams(params);

	OgrePathMapRenderTechnique* result = new OgrePathMapRenderTechnique(
												pass,
												parentRenderable,
												parentTechniqueGroup);	

	return result;
}

