#include "OgrePMWeightComputeRenderingRun.h"
#include "OgreIlluminationManager.h"
#include "OgrePMEntryPointMapRenderingRun.h"

OgrePMWeightComputeRenderingRun::OgrePMWeightComputeRenderingRun(String name, String LightName)
														   : OgreRenderingRun(1, 1)
														   , RenderingRun(1, 1)
{
	this->name = name;
	weights = 0;
	
	this->light = Root::getSingleton()._getCurrentSceneManager()->getLight(LightName);
	createWeightMap();

	OgreIlluminationManager::getSingleton().createGlobalRun(ILLUMRUN_PM_ENTRYPOINTMAP);
}

void OgrePMWeightComputeRenderingRun::createWeightMap()
{
	
	unsigned int entryPointCnt = OgreIlluminationManager::getSingleton().getPathMapEntryPoints().size();
	int width = entryPointCnt / 4096;
	int height = 4096;
	TexturePtr texPtr = Ogre::TextureManager::getSingleton().createManual(name, 
																		"default",
																		TEX_TYPE_2D,
																		width,
																		height,
																		0,
																		0,
																		PF_FLOAT32_R,
																		TU_RENDERTARGET);
	 weightTexture = texPtr.getPointer();
	 //add viewport to rendertarget
	 HardwarePixelBuffer* hpb = (weightTexture->getBuffer()).getPointer();
	 RenderTarget* rt = hpb->getRenderTarget();
	 Camera* wc = Root::getSingleton()._getCurrentSceneManager()->createCamera(name + "_CAMERA"); 
	 Viewport* v = rt->addViewport(wc);
	 v->setOverlaysEnabled(false);
	 rt->setAutoUpdated(false);
}



void OgrePMWeightComputeRenderingRun::updateFrame(unsigned long frameNum)
{
	switch(light->getType())
	{
		case Light::LT_SPOTLIGHT:

			MaterialPtr mat = MaterialManager::getSingleton().getByName("GTP/PathMap_ComputeWeights");

			GpuProgramParameters* Fparams = mat->getTechnique(0)->getPass(0)->getFragmentProgramParameters().getPointer();
			
			Vector3 lightPos = light->getPosition();
			Vector3 lightDir = light->getDirection();
		//	Radian lightAngle = light->getSpotlightOuterAngle();
			unsigned int entryPointCnt = OgreIlluminationManager::getSingleton().getPathMapEntryPoints().size();
			int col = entryPointCnt / 4096; 

			//Fparams->setNamedConstant("lightTransform", lightMatrix);
			Fparams->setNamedConstant("nRadionColumns", col);
			Fparams->setNamedConstant("lightPos", lightPos);
			Fparams->setNamedConstant("lightDir", lightDir);

			OgrePMEntryPointMapRenderingRun* PMEPrun = (OgrePMEntryPointMapRenderingRun*)
					OgreIlluminationManager::getSingleton().getGlobalRun(ILLUMRUN_PM_ENTRYPOINTMAP)->asOgreRenderingRun();
			mat->getTechnique(0)->getPass(0)->getTextureUnitState(0)->setTextureName(
						PMEPrun->getEntryPointTextureName());
				
			RenderTarget* rt = weightTexture->getBuffer().getPointer()->getRenderTarget();
			renderFullscreenQuad(mat->getName(), rt);

			//rt->writeContentsToFile("focusingmap.dds");
		
		
			break;		
	}

	unsigned int clustercount = OgreIlluminationManager::getSingleton().getPathMapClusterLengthsSize();
	if(weights == 0)
		weights = new float[clustercount];
	
	unsigned int entryPointCnt = OgreIlluminationManager::getSingleton().getPathMapEntryPoints().size();
	int width = entryPointCnt / 4096;
	int height = 4096;
	unsigned int buffersize = width * height;
	float* allEntryWeights = new float[buffersize];
	PixelBox lockBox(width, height, 1, PF_FLOAT32_R, allEntryWeights);
	weightTexture->getBuffer()->blitToMemory(lockBox);	

	int iRadion = 0;
	for(int iCluster=0; iCluster < clustercount; iCluster++)
	{
		weights[iCluster] = 0.0;
		for(int clusterSummer=0;
			clusterSummer < OgreIlluminationManager::getSingleton().getPathMapClusterLength(iCluster);
			clusterSummer++, iRadion++)
		{
			weights[iCluster] += allEntryWeights[iRadion];			
		}
		if(weights[iCluster] <= 0)
			weights[iCluster] /= OgreIlluminationManager::getSingleton().getPathMapClusterLength(iCluster); //(double)clusterLenghts[iCluster];
		else
			weights[iCluster] = 0.0f;		
	}
}


